<?php

namespace Drupal\field_extra\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns a table with all private fields.
 */
class FieldExtraController extends ControllerBase {

  /**
   * Returns the entity_type.manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Construct FieldExtraController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Provides an interface for entity type managers.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Show all private fields.
   */
  public function fields() {
    $header = [
      ['data' => $this->t('Field name')],
      ['data' => $this->t('Entity type')],
      ['data' => $this->t('Entity bundle')],
      ['data' => $this->t('Operations')],
    ];

    $rows = [];
    $field_configs = $this->entityTypeManager->getStorage('field_config')->loadMultiple();
    $config = $this->config('field_extra.private_fields');

    /** @var \Drupal\field\Entity\FieldConfig $field_config */
    foreach ($field_configs as $field_config) {
      $entity_type = $field_config->getTargetEntityTypeId();

      if (!empty($config->get('types')[$entity_type]) &&
        $settings = $field_config->getThirdPartySettings('field_extra')
      ) {
        if (!empty($settings['private_field'])) {
          $rows[] = [
            $field_config->label(),
            $entity_type,
            $field_config->getTargetBundle(),
            $field_config->toLink($this->t('Edit'), "{$entity_type}-field-edit-form"),
          ];
        }
      }
    }

    return [
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('There are no private fields.'),
    ];
  }

}
